/*
 * Copyright (C) 2003  Bradley A. Smith
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.util.Enumeration;
import java.util.Vector;

/**
 * @author Bradley A. Smith (bradley@baysmith.com)
 */
class Molecule {

  double energy;

  private Vector atoms;

  private Vector frequencies;

  Molecule() {
    atoms = new Vector();
    frequencies = new Vector();
  }

  void addAtom(Atom a) {
    atoms.addElement(a);
  }

  Atom getAtom(int index) {
    return (Atom) atoms.elementAt(index);
  }

  int getNumberAtoms() {
    return atoms.size();
  }

  Enumeration getAtoms() {
    return atoms.elements();
  }

  void removeAtoms() {
    atoms.removeAllElements();
  }

  void addFrequency(Frequency f) {
    frequencies.addElement(f);
  }

  Frequency getFrequency(int index) {
    return (Frequency) frequencies.elementAt(index);
  }

  int getNumberFrequencies() {
    return frequencies.size();
  }

  Enumeration getFrequencies() {
    return frequencies.elements();
  }

  void removeFrequencies() {
    frequencies.removeAllElements();
  }

  void removeFrequency(int index) {
    frequencies.removeElementAt(index);
  }


  Vector vibration(int frequencyIndex, int numberFrames, double freqScale,
      boolean palindrome) {

    Vector vib = new Vector(numberFrames);
    Frequency freq = (Frequency) frequencies.elementAt(frequencyIndex);
    for (int j = 0; j < numberFrames; ++j) {
      Molecule mol = new Molecule();
      mol.energy = energy;
      double scaling;
      if (palindrome) {
        scaling = freqScale * Math.cos(Math.PI * j / (numberFrames - 1));
      } else {
        scaling = freqScale * Math.sin(2.0 * Math.PI * j / numberFrames);
      }
      for (int i = 0; i < atoms.size(); i++) {

        // Modify coordinates along normal modes
        Atom a = new Atom();
        Atom atom = (Atom) atoms.elementAt(i);

        a.atomicNumber = atom.atomicNumber;
        Tuple3d t1 = new Tuple3d(freq.getVector(i));
        t1.scale(scaling);
        a.position = new Point3d(atom.position);
        a.position.add(t1);
        mol.atoms.addElement(a);
      }
      vib.addElement(mol);
    }
    return vib;
  }
}

