/*
 * Copyright (C) 2003  Center for Scientific Computing Ltd
 * Copyright (C) 2003  Bradley A. Smith
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;

/**
 * Determines the file type and returns the appropriate
 * <code>MoleculeReader</code>.
 * @author Eero Häkkinen (ehakkine@csc.fi)
 * @author Bradley A. Smith (bradley@baysmith.com)
 */
class ReaderFactory {

  static MoleculeReader createReader(Reader input) {

    BufferedReader buffer = new BufferedReader(input);
    String line;
    try {
      while (buffer.ready()) {
        line = buffer.readLine();
        if (line.indexOf("Gaussian 03:") >= 0) {
          return new Gaussian03Reader(buffer);
        } else if (line.indexOf("Gaussian 98:") >= 0) {
          return new Gaussian98Reader(buffer);
        } else if (line.indexOf("Gaussian 95:") >= 0) {
          return new Gaussian94Reader(buffer);
        } else if (line.indexOf("Gaussian 94:") >= 0) {
          return new Gaussian94Reader(buffer);
        } else if (line.indexOf("Gaussian 92:") >= 0) {
          return new Gaussian92Reader(buffer);
        } else if (line.indexOf("Gaussian G90") >= 0) {
          return new Gaussian90Reader(buffer);
        } else if (line.indexOf("GAMESS") >= 0) {
          return new GamessReader(buffer);
        } else if (line.indexOf("ACES2") >= 0) {
          return new Aces2Reader(buffer);
        } else if (line.indexOf("Amsterdam Density Functional") >= 0) {
          return new ADFReader(buffer);
        } else if (line.indexOf("DALTON") >= 0) {
          return new DaltonReader(buffer);
        } else if (line.indexOf("Jaguar") >= 0) {
          return new JaguarReader(buffer);
        } else if (line.indexOf("MOPAC") >= 0) {
          return new MopacReader(buffer);
        } else if (line.indexOf("HyperChem") >= 0) {
          return new HyperchemReader(buffer);
        } else if (line.indexOf("SPARTAN SEMIEMPIRICAL PROGRAM") >= 0) {
          return new SpartanReader(buffer);
        }
      }
    } catch (IOException e) {
      System.err.println(e);
    }
    return null;
  }
}

